## import data visualization library matplotlib and seaborn
import pandas as pd
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator, FormatStrFormatter, AutoMinorLocator, FuncFormatter
from matplotlib.ticker import AutoMinorLocator, MultipleLocator, FuncFormatter
from matplotlib import rc
from mpl_toolkits.axes_grid.inset_locator import (inset_axes, InsetPosition,
                                                  mark_inset,
                                                  zoomed_inset_axes)
import matplotlib.cm as cm
import numpy as np
import matplotlib.patches as mpatches

## import module to read files
import mimetypes
import urllib
import os

#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()
# the path is where the dataset saved

path_current=path_current.replace('\\', '/')
path = path_current + '/Data/Bub/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'
class Bubble:
    def read_file(self, file):
      """Read different types of files and return pandas dataframe.

      This function can transform multiple file type such as csv/excel/text into
      a pandas dataframe. User need to input the filename such as: 'file.csv'.
      If the file type cannot be find or not support it, it will return the message.

      Args:
          file: filename of user data source

      Returns:
          data : A pandas dataframe 

      """
        # Get the file URL
      file_url = urllib.request.pathname2url(file)  # try catch
      # use mimetypes package to guess the file type
      # For example: 'file.csv' will return 'application/vnd.ms-excel'
      ftype = mimetypes.guess_type(file_url, strict=True)[0]
        ## read data file according to its formate, default includes three types of files: csv/excel/text
        # read csv format data
      if 'excel' in ftype:
          data = pd.read_csv(path + file)
        # read excel format data
      elif 'sheet' in ftype:
          data = pd.read_excel(path + file)
        # read text format data from
      elif ftype == 'text/plain':
          data = pd.read_csv(path + file, sep="\t")
      else:
          print("File type cannot find!")
      return data


    def bubble_plot(self, file, x_col_name, y_col_name, bubble_col, x_label,
                    y_label, bubble_name, legend_title,output_name, **kwargs):
        """Bubble chart with each bubble name and sizes 

        This function will convert the user's incoming data set into DataFrame and then draw
        the Bubble Chart using Matplotlib. The bubble chart can show x, y, the size of bubbles and the names of bubbles.
        
        To use this function, you need a dataset with four variables columns. The four variables are x_col_name,
        y_col_name, bubble_col and bubble_name. The bubble_col is the column decide bubble sizes and bubble_name 
        is will show each bubble name in the legend lables.
        
        In our example, The x_col_name is life_Expectancy, The y_col_name is Birth Rate, The bubble_col is GDP and
        bubble_name is country Name. 

        Args:
            file: file name of your data source, it support csv/excel,text. for example: 'new_file.csv'
            x_col_name: The X column name
            y_col_name: The Y column name
            bubble_col : The bubble values column
            x_label: x axis label
            y_label: y axis label
            bubble_name: The column of each bubble name, it will show each bubble name in legend lables
            legend title: customize legend lables title
            plot_title: The title of the plot
            output_name : output file name


        """
        conf = {
            'plotwidth':
            8,  #weight
            'plotheight':
            6,  #height
            'my_font':
            'DejaVu Sans',
            'backgrid':
            True,
            'isframe':
            True,
            'linewidth':
            2,
            'gridlinewidth':
            0.5,
            'gridline_style':
            '--',
            'gridline_color':
            'gray',
            'gridline_alpha':
            0.8,
            'labeltext_size':
            20,
            'labelpad':
            10,
            'legend_size':
            10,
            'legend_loc':
            'upper right',
            'title_pad':
            10,
            'title_size':
            15,
            'bubble_scale':
            50,
            'title_loc':
            'center',
            'xtrick_fontsize':
            15,
            'ytrick_fontsize':
            15,
            'bubble_marker':
            'o',
            'alpha':
            0.8,
            'bubble_colors': [
                "#3d405b", "#e07a5f", "#f4f1de", "#81b29a", "#f2cc8f",
                "#264653", "#2a9d8f", "#e9c46a", "#f4a261", "#e76f51",
                '#ECE133', '#56B4E9'
            ],
            'bubblesize_legend_loc': 'lower left',
            'bubbleszie_legend_title': 'Sizes',
            'plot_title':''
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)
        ## create figure and set figure size
        fig, ax = plt.subplots(figsize=(conf['plotwidth'], conf['plotheight']))
        ## background grid setting
        if conf['backgrid'] == True:
            ax.grid(linestyle=conf['gridline_style'],
                    linewidth=conf['gridlinewidth'],
                    color=conf['gridline_color'],
                    alpha=conf['gridline_alpha'])

        ## x, y axis setting
        # fontsize: x, y title size
        ax.set_xlabel(x_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])
        ax.set_ylabel(y_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])

        # Input the valid filename and return the pandas dataframe for drawing 
        data = self.read_file(file)

        # start plot

        # origin bubble data
        bubble_list = list(data[bubble_col])

        # scale the bubble size become smaller/bigger depend on user input, the inital setting is 50
        bubble_scale = []
        for i in range(len(bubble_list)):
            bubble_scale.append(bubble_list[i] / conf['bubble_scale'])

        # bubble colors
        # count the number of x column values and match with our bubble color list
        num_index = len(data[x_col_name])
        colors = conf['bubble_colors'][:num_index]

        # plot bubble chart
        scatter = ax.scatter(x=x_col_name,
                             y=y_col_name,
                             c=colors,
                             s=bubble_scale,
                             marker=conf['bubble_marker'],
                             alpha=conf['alpha'],
                             data=data,
                             label=colors)

        # add bubble_name into the legend 
        legend_list = list(data[bubble_name])

        # match the color of the label
        l = []
        for i in range(0, num_index):
            l.append(
                mpatches.Patch(color=colors[i],
                               alpha=conf['alpha'],
                               label=legend_list[i]))
        # legend
        legend1 = ax.legend(handles=l, title=legend_title,fontsize=conf['legend_size'])

        # add the second legend(bubble_sizes)
        ax.add_artist(legend1)

        # create the handles and labels for the bubble sizes legend
        handles, labels = scatter.legend_elements(prop="sizes",
                                                  alpha=conf['alpha'])

        # scale back the origin data into the legend labels
        kw = dict(prop="sizes", func=lambda s: s * conf['bubble_scale'])

        # bubble sizes legend add into the plot
        legend2 = ax.legend(*scatter.legend_elements(**kw),
                            loc=conf['bubblesize_legend_loc'],
                            title=conf['bubbleszie_legend_title'],
                            fontsize=conf['legend_size'])

        # xtricks and yticks font size
        plt.xticks(fontsize=conf['xtrick_fontsize'])
        plt.yticks(fontsize=conf['ytrick_fontsize'])

        # plot title
        plt.title(conf['plot_title'], fontsize=conf['title_size'])

        # save the image
        plt.savefig(path_img + output_name)
        plt.show()


    def bubble_category(self, file, x_col_name, y_col_name, bubble_col,
                        x_label, y_label, category_col, output_name, **kwargs):
      
      """Bubble chart with different category 

       This function will convert the user's incoming data set into DataFrame and then draw
       the Bubble Chart using Matplotlib. The bubble chart can show x, y, the size of bubbles and the bubble category.
       Bubble category is used for when the dataset have a column with multiple type. For example, in our example
       continent is a good category. It have Asia, Europe, Africa, Oceania and Americas. Each category will have differnt
       bubble colors and also legend for different category.

       In the example, x_col_name is life_Exp, y_col_name is population, bubble_col is gdp and the category column is 
       continent. 

       Args:
            file: file name of your data source, it support csv/excel,text. for example: 'new_file.csv'
            x_col_name: The X column name
            y_col_name: The Y column name
            bubble_col : The bubble values column
            category_col: The category column of bubble
            x_label: x axis label
            y_label: y axis label
            legend title: customize legend lables title
            plot_title: The title of the plot
            output_name : output file name

      """
      conf = {
          'plotwidth':
          8,  #weight
          'plotheight':
          6,  #height
          'my_font':
          'DejaVu Sans',
          'backgrid':
          True,
          'isframe':
          True,
          'linewidth':
          2,
          'gridlinewidth':
          0.5,
          'gridline_style':
          '--',
          'gridline_color':
          'gray',
          'gridline_alpha':
          0.8,
          'labeltext_size':
          20,
          'labelpad':
          10,
          'legend_size':
          12,
          'legend_loc':
          'upper right',
          'title_pad':
          10,
          'title_size':
          15,
          'bubble_scale':
          50,
          'title_loc':
          'center',
          'xtrick_fontsize':
          15,
          'ytrick_fontsize':
          15,
          'bubble_marker':
          'o',
          'alpha':
          0.8,
          'bubble_colors': [
                "#3d405b", "#e07a5f", "#f4f1de", "#81b29a", "#f2cc8f",
                "#264653", "#2a9d8f", "#e9c46a", "#f4a261", "#e76f51",
                '#ECE133', '#56B4E9',"#3d405b", "#e07a5f", "#f4f1de", 
                "#81b29a", "#f2cc8f","#264653", "#2a9d8f", "#e9c46a",
                 "#f4a261", "#e76f51",'#ECE133', '#56B4E9'
            ],
          'plot_title':''
      }

      # when new configuraton is set, update the original one
      conf.update(kwargs)
      ## create figure and set figure size
      fig, ax = plt.subplots(figsize=(conf['plotwidth'], conf['plotheight']))
      ## background grid setting
      if conf['backgrid'] == True:
          ax.grid(linestyle=conf['gridline_style'],
                  linewidth=conf['gridlinewidth'],
                  color=conf['gridline_color'],
                  alpha=conf['gridline_alpha'])

      ## x, y axis setting
      # fontsize: x, y title size
      ax.set_xlabel(x_label,
                    fontproperties=conf['my_font'],
                    fontsize=conf['labeltext_size'],
                    labelpad=conf['labelpad'])
      ax.set_ylabel(y_label,
                    fontproperties=conf['my_font'],
                    fontsize=conf['labeltext_size'],
                    labelpad=conf['labelpad'])

      # Input the valid filename and return the pandas dataframe for drawing
      data = self.read_file(file)

      # origin bubble data
      bubble_list = list(data[bubble_col])

      # scale the bubble size become smaller
      bubble_scale = []
      for i in range(len(bubble_list)):
          bubble_scale.append(bubble_list[i] / conf['bubble_scale'])

      map_color = []
      # create the list to get the category unique value
      # Example: ['Asia' 'Europe' 'Africa' 'Americas' 'Oceania']
      category_list = data[category_col].unique()

      # map each category item to one color
      # {'Asia': 'r', 'Europe': 'b', 'Africa': 'y','Americas':'g','Oceania':'c'}
      map_color = dict(
          zip(category_list, conf['bubble_colors'][:len(category_list)]))

      # color for each item
      colors = data[category_col].map(map_color)

      # plot the bubble chart
      scatter = ax.scatter(data[x_col_name],
                            data[y_col_name],
                            s=bubble_scale,
                            color=colors,
                            label=colors,
                            alpha=conf['alpha'],
                            edgecolors='grey')

      # create the mptaches list to for legend
      match_color = []
      for key, value in map_color.items():
          match_color.append(mpatches.Patch(color=value, label=key))
      # crate the legend
      ax.legend(handles=match_color,fontsize=conf['legend_size'])

      # xtricks and yticks font size
      plt.xticks(fontsize=conf['xtrick_fontsize'])
      plt.yticks(fontsize=conf['ytrick_fontsize'])

      # title
      plt.title(conf['plot_title'], fontsize=conf['title_size'])

      # save the plot
      plt.savefig(path_img + output_name)


    def bubble_colormap(self, file, x_col_name, y_col_name, bubble_col,
                        x_label, y_label, output_name, **kwargs):
      """Bubble chart with colormap

      This function will convert the user's incoming data set into DataFrame and then draw
      the Bubble Chart using Matplotlib. The bubble chart can show x, y and bubbles colormap.

      In this bubble chart, the bubble size are fixed. The bubble_col values will decide the The depth
      of color of the bubbles. The colormap legend will place at the right hand side of the plot.

      Args:
          file: file name of your data source, it support csv/excel,text. for example: 'new_file.csv'
          x_col_name: The X column name
          y_col_name: The Y column name
          bubble_col : The bubble values column
          x_label: x axis label
          y_label: y axis label
          plot_title: The title of the plot
          output_name : output file name

      """
      conf = {
          'plotwidth':
          8,  #weight
          'plotheight':
          6,  #height
          'my_font':
          'DejaVu Sans',
          'backgrid':
          True,
          'isframe':
          True,
          'linewidth':
          2,
          'gridlinewidth':
          0.5,
          'gridline_style':
          '--',
          'gridline_color':
          'gray',
          'gridline_alpha':
          0.8,
          'labeltext_size':
          20,
          'labelpad':
          10,
          'legend_size':
          12,
          'legend_loc':
          'upper right',
          'title_pad':
          10,
          'title_size':
          15,
          'bubble_scale':
          50,
          'title_loc':
          'center',
          'xtrick_fontsize':
          15,
          'ytrick_fontsize':
          15,
          'bubble_marker':
          'o',
          'alpha':
          0.8,
          'bubble_colors': [
                "#3d405b", "#e07a5f", "#f4f1de", "#81b29a", "#f2cc8f",
                "#264653", "#2a9d8f", "#e9c46a", "#f4a261", "#e76f51",
                '#ECE133', '#56B4E9'
            ],
          'bubble_size':
          400,
          'bubble_cmap':
          'Reds',
          'plot_title':'',
          'colorbar_labelsize':15
      }

      # when new configuraton is set, update the original one
      conf.update(kwargs)
      ## create figure and set figure size
      fig, ax = plt.subplots(figsize=(conf['plotwidth'], conf['plotheight']))
      ## background grid setting
      if conf['backgrid'] == True:
          ax.grid(linestyle=conf['gridline_style'],
                  linewidth=conf['gridlinewidth'],
                  color=conf['gridline_color'],
                  alpha=conf['gridline_alpha'])

      ## x, y axis setting
      # fontsize: x, y title size
      ax.set_xlabel(x_label,
                    fontproperties=conf['my_font'],
                    fontsize=conf['labeltext_size'],
                    labelpad=conf['labelpad'])
      ax.set_ylabel(y_label,
                    fontproperties=conf['my_font'],
                    fontsize=conf['labeltext_size'],
                    labelpad=conf['labelpad'])

      # Input the valid filename and return the pandas dataframe for drawing
      data = self.read_file(file)

      # bubble colors
      num_index = len(data[x_col_name])
      colors = conf['bubble_colors'][:num_index]

      # bubble chart
      plt.scatter(x=x_col_name,
                  y=y_col_name,
                  c=bubble_col,
                  s=conf['bubble_size'],
                  cmap=conf['bubble_cmap'],
                  marker=conf['bubble_marker'],
                  alpha=conf['alpha'],
                  data=data)

      # create the colorbar 
      cbar = plt.colorbar(ax=ax)
      cbar.ax.tick_params(labelsize=conf['colorbar_labelsize'])

      # xtricks and yticks font size
      plt.xticks(fontsize=conf['xtrick_fontsize'])
      plt.yticks(fontsize=conf['ytrick_fontsize'])

      # title
      plt.title(conf['plot_title'], fontsize=conf['title_size'])

      # save the plot
      plt.savefig(path_img + output_name)
      plt.show()



import pandas as pd
import numpy as np

# Data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager

import mimetypes
import urllib
import os


#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Hea/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class heat_map:
    # read data
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]
    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  # check the available file name
  # if the input file name already existed then rename to file_1, file_2
  def get_available_name(self,filename):
    n=[1]
    def check_meta(file_name):
      file_name_new=file_name
      if file_name in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
          file_name_new=file_name+'_'+str(n[0])
          n[0]+=1
      if file_name_new in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
          file_name_new=check_meta(file_name)
      return file_name_new
    available_name=check_meta(filename)
    return available_name
  # file: file name of your data source
  # row_labels: the x axis labels
  # col_labels: the y axis labels
  # paper_type : 'single' or 'double'
  def heat(self, file, row_labels, col_labels, paper_type, **kwargs):
    # Configuration of the heatmap
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # my_font: the typeface of x, y labels
    # labeltext_size: text size of x,y labels
    # labelpad: pad size of label
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # cmap： the color of heatmap
    # annotate: True or False, if true, the value will show in the heatmap
    # colorbar: True or False, if true, the color bar of heatmap will show
    # barlabel: the label of color bar
    # xlabel_rotate: the rotation of x axis labels
    # ylabel_rotate: the rotation of y axis labels
    # barlabel_rotate: the rotation of colorbar labels
    # x_label: the content of x label
    # y_label: the content of y label
    # annotate_textsize: the font size of annotation
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    # file_name: the file name in saving image
    single_column_conf={ 'plotwidth':8,
                      'plotheight':6, 
                      'my_font':'DejaVu Sans',
                      'labeltext_size':20,
                      'labelpad':10,
                      'title':False,
                      'title_pad':10,
                      'title_size':20,
                      'title_loc':'center', 
                      'cmap':'GnBu',
                      'alpha':0.9,
                      'annotate':True,
                      'colorbar':True,
                      'barlabel':None,
                      'xlabel_rotate':45,
                      'ylabel_rotate':0,
                      'barlabel_rotate':-90,
                      'x_label':None,
                      'y_label':None,
                      'annotate_textsize':13,
                      'save_image':False,
                      'savefig_bbox_inches':'tight',
                      'file_name':'heatmap'
    }

    double_column_conf={ 'plotwidth':8,
                      'plotheight':6,
                      'my_font':'DejaVu Sans',
                      'backgrid':True,
                      'isframe':True,
                      'labeltext_size':22,
                      'labelpad':10,
                      'title':False,
                      'title_pad':10,
                      'title_size':22,
                      'title_loc':'center', 
                      'cmap':'GnBu',
                      'alpha':0.9,
                      'annotate':True,
                      'colorbar':True,
                      'barlabel':None,
                      'xlabel_rotate':45,
                      'ylabel_rotate':0,
                      'barlabel_rotate':-90,
                      'x_label':None,
                      'y_label':None,
                      'annotate_textsize':12,
                      'save_image':False,
                      'savefig_bbox_inches':'tight',
                      'file_name':'heatmap'
    }

    # choose a configuration
    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf  

    # when new configuraton is set, update the original one
    conf.update(kwargs)  
    # create figure and set figure size  
    fig, ax_left = plt.subplots(figsize = (conf['plotwidth'], conf['plotheight']))

    # x,y label setting                   
    ax_left.set_xlabel(conf['x_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    ax_left.set_ylabel(conf['y_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    
    # read file 
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name')
    

    # plot
    # find the row_label
    if row_labels not in list(data.columns):
      k=''
      for i in data.columns:
        if set(row_labels) <= set(data[i].values): 
          k=i
        else:
          pass

      data=data.set_index(k)
      data.index.name=None
    # plot heatmap        
    im=ax_left.imshow(data,cmap=conf['cmap'],alpha=conf['alpha'])
    ax_left.set_xticks(np.arange(len(data.columns)))
    ax_left.set_yticks(np.arange(len(data.index)))
    # and label them with the respective list entries
    ax_left.set_xticklabels(data.columns,fontsize=conf['labeltext_size'])
    ax_left.set_yticklabels(data.index,fontsize=conf['labeltext_size'])
    # rotate x, y axis
    plt.setp(ax_left.get_xticklabels(), rotation=conf['xlabel_rotate'], ha="right",
          rotation_mode="anchor")
    plt.setp(ax_left.get_yticklabels(), rotation=conf['ylabel_rotate'], ha="right",
          rotation_mode="anchor")
    # if colorbar is true, draw color bar
    if conf['colorbar'] == True:
      cbar = ax_left.figure.colorbar(im, ax=ax_left)
      cbar.ax.tick_params(labelsize=conf['labeltext_size'])
      if conf['barlabel'] != None:
        cbar.ax.set_ylabel(conf['barlabel'], rotation=conf['barlabel_rotate'], va="bottom",fontsize=conf['labeltext_size'])
    # define an annotate function
    # the default format is .2f
    # if the background color is light the font color will be black, else, the font color will be white
    def annotate_heatmap(im, data=None, valfmt="{x:.2f}",
                     textcolors=("black", "white"),
                     threshold=None, **textkw):
      if not isinstance(data, (list, np.ndarray)):
          data = im.get_array()
      # get the boundary threshold （use 1/2 of max)
      if threshold is not None:
        threshold = im.norm(threshold)
      else:
        threshold = im.norm(data.max())/2.
      # set the position of annotation  
      kw = dict(horizontalalignment="center",
                  verticalalignment="center")
      kw.update(textkw)

      if isinstance(valfmt, str):
        valfmt = matplotlib.ticker.StrMethodFormatter(valfmt)
      texts = []
      # get the colored annotate font
      for i in range(data.shape[0]):
        for j in range(data.shape[1]):
          kw.update(color=textcolors[int(im.norm(data[i, j]) > threshold)])
          text = im.axes.text(j, i, valfmt(data[i, j]), fontsize=conf['annotate_textsize'],**kw)
          texts.append(text)
      return texts

    if conf['annotate'] == True:
      texts = annotate_heatmap(im, valfmt="{x:.1f}")
    # title setting
    if conf['title'] == False:
      pass
    else:
      ax_left.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])
    # save image setting
    if conf['save_image'] == True:
      file_name=conf['file_name']
      file_newname = self.get_available_name(file_name)
      plt.savefig(path_img+file_newname, bbox_inches=conf['savefig_bbox_inches'],dpi=600,format='jpg') 

    plt.show()


import pandas as pd
import numpy as np

# Data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager

import mimetypes
import urllib
import os


#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/His/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class hist_plot:
    # read data
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]
    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  # check the available file name
  # if the input file name already existed then rename to file_1, file_2
  def get_available_name(self,filename):
    n=[1]
    def check_meta(file_name):
        file_name_new=file_name
        if file_name in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=file_name+'_'+str(n[0])
            n[0]+=1
        if file_name_new in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=check_meta(file_name)
        return file_name_new
    available_name=check_meta(filename)
    return available_name

  # file: file name of your data source
  # x_col_name: ['x_column_name_a','x_column_name_b'...]
  # paper_type : 'single' or 'double'
  def hist(self,file ,x_col_name,paper_type ,**kwargs):
    # Configuration of the histogram chart
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # my_font: the typeface of x, y labels
    # backgrid: backgrid of the plot
    # isframe: frame of the plot
    # labeltext_size: text size of x,y labels
    # labelpad: pad size of label
    # gridlinewidth: the width of grid
    # linewidth: the line width of each bin
    # legend_size: size of legend
    # legend_loc: location of legend
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # x_label: the content of x label
    # y_label: the content of y label
    # color: the color of histogram
    # edgecolor: the color of bins outlines
    # alpha: the alpha of bins
    # histtype: the type of histogram
    # orientation: vertical or horizontal
    # stacked: True or False, if true, draw stack histogram
    # bins: set the number of bins
    # density: True or False, if true, the y axis means the density of each bins
    # density_line: True or False, if true, draw the density line of histogram
    # tick_size: set the size of ticks
    # tick_direction: set the direction of ticks
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    # file_name: the file name in saving image

    single_column_conf={ 'plotwidth':8,
                        'plotheight':6, 
                        'my_font':'DejaVu Sans',
                        'backgrid':True,
                        'isframe':True,
                        'labeltext_size':25,
                        'labelpad':10,
                        'gridlinewidth':0.5,
                        'linewidth':2,
                        'legend_size':15,
                        'legend_loc':'upper right', 
                        'title':False,
                        'title_pad':10,
                        'title_size':20,
                        'title_loc':'center', 
                        'x_label':None,
                        'y_label':None, 
                        'color':sns.color_palette("hls", 8),
                        'edgecolor':'black',
                        'alpha':0.7,
                        'orientation':'vertical',
                        'histtype':'bar',
                        'stacked':False,
                        'bins':None,
                        'density':False,
                        'density_line':False,
                        'tick_size':20,
                        'tick_direction':'out',
                        'save_image':False,
                        'savefig_bbox_inches':'tight',
                        'file_name':'histogram'                    
                        }
    double_column_conf={ 'plotwidth':8,
                        'plotheight':6, 
                        'my_font':'DejaVu Sans',
                        'backgrid':True,
                        'isframe':True,
                        'labeltext_size':26,
                        'labelpad':10,
                        'gridlinewidth':0.5,
                        'linewidth':2,
                        'legend_size':17,
                        'legend_loc':'upper right', 
                        'title':False,
                        'title_pad':10,
                        'title_size':21,
                        'title_loc':'center', 
                        'x_label':None,
                        'y_label':None, 
                        'color':sns.color_palette("hls", 8),
                        'edgecolor':'black',
                        'alpha':0.7,
                        'orientation':'vertical',
                        'histtype':'bar',
                        'stacked':False,
                        'bins':None,
                        'density':False,
                        'density_line':False,
                        'tick_size':21,
                        'tick_direction':'out',
                        'save_image':False,
                        'savefig_bbox_inches':'tight',
                        'file_name':'histogram'                      
                        }
                   
    # choose configuration
    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf  

    # when new configuraton is set, update the original one
    conf.update(kwargs)  

    # create figure and set figure size  
    fig, ax_left = plt.subplots(figsize = (conf['plotwidth'], conf['plotheight']))
    # background grid setting
    if conf['backgrid'] == True:
      ax_left.grid(linestyle="--", linewidth=conf['gridlinewidth'], color='gray', alpha=0.5)                    
    # x,y label setting
    ax_left.set_xlabel(conf['x_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    ax_left.set_ylabel(conf['y_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    
    # read file 
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name')
   
    #plot
    # single column histogram
    if len(x_col_name) == 1:
      pal=conf['color']
      n, bins, patches=ax_left.hist(data[x_col_name[0]],bins=conf['bins'],density=conf['density'],alpha=conf['alpha'],
                                    orientation=conf['orientation'],histtype=conf['histtype'],stacked=conf['stacked'],
                                    )      
      pal=sns.color_palette('hls',len(patches))
      # change the color of bins
      for i in range(len(patches)):
        patches[i].set_color(pal[i])
        patches[i].set_edgecolor('black')
      # if density is true, the histogram means density, if density_line is True, draw density line
      if conf['density'] ==True and conf['density_line']==True:
        # get the density line
        mu=np.mean(data[x_col_name[0]])
        sigma=np.std(data[x_col_name[0]])
        y = ((1 / (np.sqrt(2 * np.pi) * sigma)) *
          np.exp(-0.5 * (1 / sigma * (bins - mu))**2))
        ax_left.plot(bins,y,'-',color='black',linewidth=1)
    # multi columns histogram (separate simple histogram or stacked histogram)
    if len(x_col_name) != 1:
      # stacked histogram
      if conf['stacked'] != True:
        for i in range(len(x_col_name)):
          n, bins, patches=ax_left.hist(data[x_col_name[i]],bins=conf['bins'],density=conf['density'],alpha=conf['alpha'],
                                      color=conf['color'][i],orientation=conf['orientation'],histtype=conf['histtype'],stacked=conf['stacked'],label=x_col_name[i])
          mu=np.mean(data[x_col_name[i]])
          sigma=np.std(data[x_col_name[i]])
          for i in range(len(patches)):
            patches[i].set_edgecolor('black')  
          if conf['density'] ==True and conf['density_line']==True:
            # get density line
            y = ((1 / (np.sqrt(2 * np.pi) * sigma)) *
              np.exp(-0.5 * (1 / sigma * (bins - mu))**2))
            ax_left.plot(bins,y,'-',color='black',linewidth=1)
      # simple histogram
      else:  
        datalist=[]
        num=0
        for i in range(len(data)):
          datalist.append(data.loc[i].values)
          # set the size of palette
          if len(data.loc[i].values)>num:
            num=len(data.loc[i].values)
        data=np.array(datalist)
        # each columns has one color
        color=sns.color_palette("hls", num)
        n, bins, patches=ax_left.hist(data,bins=conf['bins'],density=conf['density'],alpha=conf['alpha'],
                                      color=color,orientation=conf['orientation'],histtype=conf['histtype'],stacked=conf['stacked'],edgecolor=conf['edgecolor'],label=x_col_name)
    
      ax_left.legend(loc=conf['legend_loc'], fontsize=conf['legend_size'])
    # frame setting
    if conf['isframe'] == False:
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
    # set x, y tick's direction, default:out, can be set to in,out,inout
    if conf['tick_direction'] == 'in':
      matplotlib.rcParams['xtick.direction'] = 'in'
      matplotlib.rcParams['ytick.direction'] = 'in'
    elif conf['tick_direction'] == 'inout':
      matplotlib.rcParams['xtick.direction'] = 'inout'
      matplotlib.rcParams['ytick.direction'] = 'inout'
    elif conf['tick_direction'] == 'out':
      matplotlib.rcParams['xtick.direction'] = 'out'
      matplotlib.rcParams['ytick.direction'] = 'out'
    # set the tick size
    for tick in ax_left.xaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    for tick in ax_left.yaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    if conf['title'] == False:
      pass
    else:
      ax_left.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])
    # save the plot
    if conf['save_image'] == True:
      file_name=conf['file_name']
      file_newname = self.get_available_name(file_name)
      plt.savefig(path_img+file_newname, bbox_inches=conf['savefig_bbox_inches'],dpi=600,format='jpg') 
     
    # showing the image
    plt.show() 


## import python data analysis library 
import pandas as pd

## import data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator, FormatStrFormatter, AutoMinorLocator, FuncFormatter
from matplotlib.ticker import AutoMinorLocator, MultipleLocator, FuncFormatter
from matplotlib import rc
from mpl_toolkits.axes_grid.inset_locator import (inset_axes, InsetPosition, mark_inset, zoomed_inset_axes)

## import module to read files
import mimetypes
import urllib
import os

# import google colab to use google colab as editor
# if you use other editor, do not need to import google.colab here
#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Line/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class line:  

  ## read file function: read three kinds of format file csv/excel/text
  # if you have other format of file, please change the function manually
  # file: str, filename (e.g.'Vertical_Bar.txt')
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]

    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
  
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  ## line chart plotting function
  # file: file name of your data source
  # x_col_name: ['index'] or ['x_column_name_a','x_column_name_b'...]
  # y_col_name: ['y_column_name_a','y_column_name_b'...]
  # x_label: x axis label 
  # y_label: y axis label
  # legend_label: legend labels names
  # paper_type: 'single' or 'double'
  

  def plot(self, file, x_col_name, y_col_name, x_label, y_label, legend_label, paper_type, **kwargs):

    ## Configuration of the line chart
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # backgrid: backgrid of the plot
    # isframe: frame of the plot
    # my_font: the typeface of x, y labels
    # linewidth: linewidth of the lines in the plot
    # gridlinewidth: if backgrid is True, grid linewidth is the line width of background grid
    # sa_linecolor: single column paper linecolors palette
    # da_linecolor: double columns paper linecolors palette 
    # linestyle: line shapes library 
    # labeltext_size: text size of x,y labels
    # labelpad: pad size of label
    # legend_size: size of legend
    # legend_loc: location of legend
    # ncol: number of columns of legend
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # markers: True or False as options. If it is True, add markers
    # markersize: if the title is True, modify size of marker
    # markers_shape: shapes library of marker
    # xy_lim: True or False as options. If it is True, add x and y axis' value range
    # x_range: if the xy_lim is True, set x axis range
    # y_range: if the xy_lim is True, set x axis range
    # inset: True or False as options. If it is True, add inset plot 
    # xin_start: if the inset is True, the inset plot x axis starts from xin_start
    # xin_end: if the inset is True, the inset plot x axis ends from xin_end
    # yin_start: if the inset is True, the inset plot y axis starts from yin_start
    # yin_end:  if the inset is True, the inset plot y axis starts from yin_end
    # ticks: True or False as options. If it is True, add ticks of x and y axis.
    # tick_size: size of tick 
    # tick_direction: 'out', 'inout' and 'in' options.
    # x_minor_locator: number of minor ticks in x axis
    # y_minor_locator: number of minor ticks in y axis
    # present_linevalue: True or False as options. If it is True, present point value in the line
    # double_axis: True or False as options. If it is True, add second axis on the right of figure     
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    single_column_conf={'plotwidth':8,
              'plothight':6,
              'backgrid':True,
              'isframe':True,
              'my_font':'DejaVu Sans',
              'linewidth':2,
              'gridlinewidth':0.5,              
              'sa_linecolor':['#0173B2', '#DE8F05', '#029E73', '#D55E00', 
                      '#CC78BC', '#8E5638', '#FBAFE4', '#949494', 
                      '#ECE133', '#56B4E9'],
              'da_linecolor':{'left':['#D6DEBF','#AECEA1','#82BB92','#5EA28D',
                          '#49838A','#3E5F7E','#383C65','#2B1E3E'],
                      'right':['#2C1E3D','#51315E','#764476','#9A5B88',
                          '#B77495','#CF91A3','#E0B1B4','#EDD1CB']},              
              'linestyle':['-',':','-.','--','-',':','-.','--','-','-',':','-.'],  
              'labeltext_size':18,
              'labelpad':10,
              'legend_size':10,
              'legend_loc':'upper right',
              'ncol':2,
              'title':False,
              'title_pad':10,
              'title_size':20,
              'title_loc':'center',  
              'markers':False,
              'markersize':8,
              'markers_shape':['o','v','D','X','P','2','p','x','d','4','<','*'], 
              'xy_lim':True,
              'x_range':False,
              'y_range':False,
              'xin_start':0, 
              'xin_end':0, 
              'yin_start':0, 
              'yin_end':0,
              'ticks':True,
              'tick_size':14,
              'tick_direction':'out',
              'x_minor_locator':5,
              'y_minor_locator':2, 
              'present_linevalue':False,
              'double_axis':False, 
              'inset':False,
              'save_image':False,
              'savefig_bbox_inches':'tight'}

    double_column_conf={'plotwidth':8,
              'plothight':6,
              'backgrid':True,
              'isframe':True,
              'my_font':'DejaVu Sans',
              'linewidth':2,
              'gridlinewidth':0.5,              
              'sa_linecolor':['#0173B2', '#DE8F05', '#029E73', '#D55E00', 
                      '#CC78BC', '#8E5638', '#FBAFE4', '#949494', 
                      '#ECE133', '#56B4E9'],
              'da_linecolor':{'left':['#D6DEBF','#AECEA1','#82BB92','#5EA28D',
                          '#49838A','#3E5F7E','#383C65','#2B1E3E'],
                      'right':['#2C1E3D','#51315E','#764476','#9A5B88',
                          '#B77495','#CF91A3','#E0B1B4','#EDD1CB']},              
              'linestyle':['-',':','-.','--','-',':','-.','--','-','-',':','-.'],  
              'labeltext_size':18,
              'labelpad':10,
              'legend_size':10,
              'legend_loc':'upper right',
              'ncol':2,
              'title_pad':10,
              'title':False,
              'title_size':20,
              'title_loc':'center',  
              'markers':False,
              'markersize':8,
              'markers_shape':['o','v','D','X','P','2','p','x','d','4','<','*'], 
              'xy_lim':True,
              'x_range':False,
              'y_range':False,
              'xin_start':0, 
              'xin_end':0, 
              'yin_start':0, 
              'yin_end':0,
              'ticks':True,
              'tick_size':14,
              'tick_direction':'out',
              'x_minor_locator':5,
              'y_minor_locator':2, 
              'present_linevalue':False,
              'double_axis':False, 
              'inset':False,
              'save_image':False,
              'savefig_bbox_inches':'tight'}

    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf 
    
    # when new configuraton is set, update the original one
    conf.update(kwargs)
    

    #Initializing the line
    ## plot size setting
    # figsize: the size of the line chart, (width,hight)
    fig, ax_1eft = plt.subplots(figsize =(conf['plotwidth'], conf['plothight']))
      
    ## background grid setting
    if conf['backgrid'] == True:
      ax_1eft.grid(linestyle="--", linewidth=conf['gridlinewidth'], color='gray', alpha=0.5)
    
    # read file
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name')
     
    # Plotting single axis line chart
    if conf['double_axis'] == False:
      
      ## x, y axis setting
      # fontsize: x, y title size
      # labelpad: scalar, optional, default: None 
      ax_1eft.set_xlabel(x_label, fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
      ax_1eft.set_ylabel(y_label, fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
      
      # use the plot function
      # linewidth: the line width, here set to 2
      # x column is index
      if x_col_name[0] == 'index':
        # line with markers
        if conf['markers'] == True:
          for i in range(0, len(y_col_name)):
            ax_1eft.plot(data.index, data[y_col_name[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label[i], color=conf['sa_linecolor'][i])    
        # line without markers    
        else:
          for i in range(0, len(y_col_name)):
            ax_1eft.plot(data.index, data[y_col_name[i]], linestyle=conf['linestyle'][0], 
                    linewidth=conf['linewidth'], label=legend_label[i], color=conf['sa_linecolor'][i])

      # x column is not index
      else:
        # line with markers
        if conf['markers'] == True:
          for i in range(0, len(y_col_name)):
            if len(x_col_name)==1:
              ax_1eft.plot(data[x_col_name[0]], data[y_col_name[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label[i], color=conf['sa_linecolor'][i])
            else: 
              ax_1eft.plot(data[x_col_name[i]], data[y_col_name[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label[i], color=conf['sa_linecolor'][i])
        # line without markers    
        else:
          for i in range(0, len(y_col_name)):
            if len(x_col_name)==1:
              ax_1eft.plot(data[x_col_name[0]], data[y_col_name[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                      label=legend_label[i], color=conf['sa_linecolor'][i])
            else:
              ax_1eft.plot(data[x_col_name[i]], data[y_col_name[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                      label=legend_label[i], color=conf['sa_linecolor'][i])             

    
      if conf['inset'] == True:
      
        axins = zoomed_inset_axes(ax_1eft, 3, bbox_to_anchor=(0.43,0.7), bbox_transform=ax_1eft.transAxes) 
      
        ## drawing lines of inset plot
        
        # use the plot function
        # linewidth: the line width, here set to 2
        # x column is index
        if x_col_name[0] == 'index':
          # line with markers
          if conf['markers'] == True:
            for i in range(0, len(y_col_name)):
              axins.plot(data.index, data[y_col_name[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label[i], color=conf['sa_linecolor'][i])    
          # line without markers    
          else:
            for i in range(0, len(y_col_name)):
              axins.plot(data.index, data[y_col_name[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                      label=legend_label[i], color=conf['sa_linecolor'][i])
        # x column is not index
        else:
          # line with markers
          if conf['markers'] == True:
            for i in range(0, len(y_col_name)):
              if len(x_col_name)==1: 
                axins.plot(data[x_col_name[0]],data[y_col_name[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['sa_linecolor'][i]) 
              else:
                axins.plot(data[x_col_name[i]],data[y_col_name[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['sa_linecolor'][i])   
          # line without markers    
          else:
            for i in range(0, len(y_col_name)):
              if len(x_col_name)==1: 
                axins.plot(data[x_col_name[0]],data[y_col_name[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                      label=legend_label[i], color=conf['sa_linecolor'][i])
              else:
                axins.plot(data[x_col_name[i]],data[y_col_name[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                      label=legend_label[i], color=conf['sa_linecolor'][i])

        ## inset plot x,y axis range limit
        axins.set_xlim(conf['xin_start'], conf['xin_end'])
        axins.set_ylim(conf['yin_start'], conf['yin_end']) 

    # double axis line chart
    elif conf['double_axis'] == True:
      
      ## x, y axis setting
      # fontsize: x, y title size
      # labelpad: scalar, optional, default: None 
      ax_1eft.set_xlabel(x_label, fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
      ax_1eft.set_ylabel(y_label[0], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
      
      
      # left x,y columns and labels
      x_col_name_l = x_col_name[0]
      y_col_name_l = y_col_name[0]
      legend_label_l = legend_label[0]
          
      # use the plot function
      # linewidth: the line width, here set to 2
      # x column is index  
      if x_col_name_l[0] == 'index':
        # line with markers
        
        if conf['markers'] == True:
          for i in range(0, len(y_col_name_l)):
            ax_1eft.plot(data.index, data[y_col_name_l[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])    
        # line without markers    
        else:
          for i in range(0, len(y_col_name_l)):
            ax_1eft.plot(data.index, data[y_col_name_l[i]], linestyle=conf['linestyle'][0], 
                    linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])

      # x column is not index
      else:
        # line with markers
        if conf['markers'] == True:
          for i in range(0, len(y_col_name_l)):
            if len(x_col_name_l)==1:
              ax_1eft.plot(data[x_col_name_l[0]], data[y_col_name_l[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])
            else: 
              ax_1eft.plot(data[x_col_name_l[i]], data[y_col_name_l[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                      linestyle=linestyle[0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])
        # line without markers    
        else:
          for i in range(0, len(y_col_name_l)):
            if len(x_col_name_l)==1:
              ax_1eft.plot(data[x_col_name_l[0]], data[y_col_name_l[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                      label=legend_label_l[i], color=conf['da_linecolor']['left'][i])
            else:
              ax_1eft.plot(data[x_col_name_l[i]], data[y_col_name_l[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                      label=legend_label_l[i], color=conf['da_linecolor']['left'][i])
      
      ## instantiate a left axis that shares the same x-axis
      ax_right = ax_1eft.twinx()
      
      ax_right.set_ylabel(y_label[1], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])    

      # already handled the x-label with ax_1eft
      # alpha: transparency, soft color
     
      # right x,y columns and labels
      x_col_name_r = x_col_name[1]
      y_col_name_r = y_col_name[1]
      legend_label_r = legend_label[1]

      # use the plot function
      # linewidth: the line width, here set to 2
      # x column is index
      if x_col_name_r[0] == 'index':
        # line with markers
        if conf['markers'] == True:
          for i in range(0, len(y_col_name_r)):
            ax_right.plot(data.index, data[y_col_name_r[i]], linestyle=conf['linestyle'][3], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i])    
        # line without markers    
        else:
          for i in range(0, len(y_col_name_r)):
            ax_right.plot(data.index, data[y_col_name_r[i]], linestyle=conf['linestyle'][3],  
                linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i]) 
      # x column is not index
      else:
        # line with markers
        if conf['markers'] == True:   
          for i in range(0, len(y_col_name_r)):
            if len(x_col_name_r)==1:
              ax_right.plot(data[x_col_name_r[0]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                  linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i])
            else:
              ax_right.plot(data[x_col_name_r[i]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                  linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i])
        else:
          for i in range(0, len(y_col_name_r)):
            if len(x_col_name_r)==1:
              ax_right.plot(data[x_col_name_r[0]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3],
                  linewidth=conf['linewidth'], label=legend_label_r[i],color=conf['da_linecolor']['right'][i])
            else:
              ax_right.plot(data[x_col_name_r[i]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3],  
                  linewidth=conf['linewidth'], label=legend_label_r[i],color=conf['da_linecolor']['right'][i])
      
      
      # plotting inset plot 
      if conf['inset'] == True:
      
        zoomed_left_axis = zoomed_inset_axes(ax_1eft, 1, bbox_to_anchor=(0.43,0.7), bbox_transform=ax_1eft.transAxes) 
      
        ## drawing lines of inset plot
        
        # use the plot function
        # linewidth: the line width, here set to 2
        # x column is index
        # use the plot function
        # linewidth: the line width, here set to 2
        # x column is index  
        if x_col_name_l[0] == 'index':
          # line with markers
          
          if conf['markers'] == True:
            for i in range(0, len(y_col_name_l)):
              zoomed_left_axis.plot(data.index, data[y_col_name_l[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                        linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])    
          # line without markers    
          else:
            for i in range(0, len(y_col_name_l)):
              zoomed_left_axis.plot(data.index, data[y_col_name_l[i]], linestyle=conf['linestyle'][0], 
                      linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])

        # x column is not index
        else:
          # line with markers
          if conf['markers'] == True:
            for i in range(0, len(y_col_name_l)):
              if len(x_col_name_l)==1:
                zoomed_left_axis.plot(data[x_col_name_l[0]], data[y_col_name_l[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                        linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])
              else: 
                zoomed_left_axis.plot(data[x_col_name_l[i]], data[y_col_name_l[i]], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                        linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], label=legend_label_l[i], color=conf['da_linecolor']['left'][i])
          # line without markers    
          else:
            for i in range(0, len(y_col_name_l)):
              if len(x_col_name_l)==1:
                zoomed_left_axis.plot(data[x_col_name_l[0]], data[y_col_name_l[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                        label=legend_label_l[i], color=conf['da_linecolor']['left'][i])
              else:
                zoomed_left_axis.plot(data[x_col_name_l[i]], data[y_col_name_l[i]], linestyle=conf['linestyle'][0], linewidth=conf['linewidth'], 
                        label=legend_label_l[i], color=conf['da_linecolor']['left'][i])

        ## inset plot x,y axis range limit
        zoomed_left_axis.set_xlim(conf['xin_start'], conf['xin_end'])
        zoomed_left_axis.set_ylim(conf['yin_start'], conf['yin_end'])   

        zoomed_right_axis = zoomed_left_axis.twinx()
        ## drawing lines of inset plot
        
        # use the plot function
        # linewidth: the line width, here set to 2
        # x column is index
        if x_col_name_r[0] == 'index':
          # line with markers
          if conf['markers'] == True:
            for i in range(0, len(y_col_name_r)):
              zoomed_right_axis.plot(data.index, data[y_col_name_r[i]], linestyle=conf['linestyle'][3], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                  linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i])    
          # line without markers    
          else:
            for i in range(0, len(y_col_name_r)):
              zoomed_right_axis.plot(data.index, data[y_col_name_r[i]], linestyle=conf['linestyle'][3],  
                  linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i]) 
        # x column is not index
        else:
          # line with markers
          if conf['markers'] == True:   
            for i in range(0, len(y_col_name_r)):
              if len(x_col_name_r)==1:
                zoomed_right_axis.plot(data[x_col_name_r[0]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                    linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i])
              else:
                zoomed_right_axis.plot(data[x_col_name_r[i]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3], marker=conf['markers_shape'][i], markersize=conf['markersize'], 
                    linewidth=conf['linewidth'], label=legend_label_r[i], color=conf['da_linecolor']['right'][i])
          else:
            for i in range(0, len(y_col_name_r)):
              if len(x_col_name_r)==1:
                zoomed_right_axis.plot(data[x_col_name_r[0]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3],
                    linewidth=conf['linewidth'], label=legend_label_r[i],color=conf['da_linecolor']['right'][i])
              else:
                zoomed_right_axis.plot(data[x_col_name_r[i]], data[y_col_name_r[i]], linestyle=conf['linestyle'][3],  
                    linewidth=conf['linewidth'], label=legend_label_r[i],color=conf['da_linecolor']['right'][i])

      ## presenting values on graph
      if conf['present_linevalue'] == True:
        if x_col_name[0] == 'index':
          for i in range(0, len(y_col_name)):
            for x,y in zip(data.index.values.tolist(),data[y_col_name[i]].values.tolist()):
              plt.text(x, y+0.01, str(round(y,2)), color = color[i])
      
        else:
          for i in range(0, len(y_col_name)):
            for x,y in zip(data[x_col_name[i]].values.tolist(),data[y_col_name[i]].values.tolist()):
              plt.text(x, y+0.01, str(round(y,2)), color = color[i])
      else:
        pass     
     
     
      ## presenting left_axis values on graph
      if conf['present_linevalue'] == True:
        if x_col_name_l[0] == 'index':
          for i in range(0, len(y_col_name_l)):
            for x,y in zip(data.index.values.tolist(),data[y_col_name_l[i]].values.tolist()):
              plt.text(x, y+0.01, str(round(y,2)), color = color[i])
      
        else:
          for i in range(0, len(y_col_name_l)):
            for x,y in zip(data[x_col_name_l[i]].values.tolist(),data[y_col_name_l[i]].values.tolist()):
              plt.text(x, y+0.01, str(round(y,2)), color = color[i])
      else:
        pass

      ## presenting right_axis values on graph
      if conf['present_linevalue'] == True:
        if x_col_name_r[0] == 'index':
          for i in range(0, len(y_col_name_r)):
            for x,y in zip(data.index.values.tolist(),data[y_col_name_r[i]].values.tolist()):
              plt.text(x, y+0.01, str(round(y,2)), color = color[i])
      
        else:
          for i in range(0, len(y_col_name_r)):
            for x,y in zip(data[x_col_name_r[i]].values.tolist(),data[y_col_name_r[i]].values.tolist()):
              plt.text(x, y+0.01, str(round(y,2)), color = color[i])
      else:
        pass    
    
    ## x,y axis range limitation
    if conf['x_range'] == False:
      pass
    else:
      # x axis valure start from xaxis_start value, and end in max of the file add a small range of value
      ax_1eft.set_xlim(conf['x_range'][0], conf['x_range'][1])        

    if conf['y_range'] == False:
      pass
    else:
      # y axis valure start from yaxis_start value, and end in max of the file add a small range of value
      ax_1eft.set_ylim(conf['y_range'][0], conf['y_range'][1])
  
    ## ticks setting
    if conf['ticks'] == True:
    ## major and minor ticks for x axia
    # set x axis' detailed ticks label 
      ax_1eft.xaxis.set_minor_locator(AutoMinorLocator(conf['x_minor_locator']))
    # set y axis' detailed ticks label 
      ax_1eft.yaxis.set_minor_locator(AutoMinorLocator(conf['y_minor_locator']))
    elif conf['ticks'] == False:
      pass

    ## size of numbers on the ticks of x,y axis' setting
    for tick in ax_1eft.xaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    for tick in ax_1eft.yaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size']) 



    # if False, top and right borders removing
    
    if conf['isframe'] == False:
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)

    ## set x, y tick's direction, default:out, can be set to in,out,inout
    if conf['tick_direction'] == 'in':
      matplotlib.rcParams['xtick.direction'] = 'in'
      matplotlib.rcParams['ytick.direction'] = 'in'
    elif conf['tick_direction'] == 'inout':
      matplotlib.rcParams['xtick.direction'] = 'inout'
      matplotlib.rcParams['ytick.direction'] = 'inout'
    elif conf['tick_direction'] == 'out':
      matplotlib.rcParams['xtick.direction'] = 'out'
      matplotlib.rcParams['ytick.direction'] = 'out'

    ## legend setting
    # ncol: number of legend column
    # loc: position of the legend
    ax_1eft.legend(ncol=conf['ncol'], loc=conf['legend_loc'], fontsize=conf['legend_size'])

    ## title and position
    
    if conf['title'] == False:
      pass
    else:
      ax_1eft.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])

    ## save image as pdf to path folder
    # bbox in inches, only the given portion of the figure is saved, figure out the tight bbox of the figure
    if conf['save_image'] == True:
      plt.savefig(path_img+'line_chart.pdf', bbox_inches=conf['savefig_bbox_inches'])
     
    # showing the image
    plt.show()   
